@@ -3,7 +3,6 @@
 #define CALC_SUMS_H
 
 #include <stdint.h>
-#include "librhash/rhash.h"
 #include "common_func.h"
 #include "hash_check.h"
 
@@ -24,7 +23,7 @@ struct file_info {
 	uint64_t size; /* the size of the file */
 	double time; /* file processing time in seconds */
 	struct infohash_ctx *infohash;
-	rhash rctx;  /* state of hash algorithms */
+	struct rhash_context* rctx;  /* state of hash algorithms */
 	int error;  /* -1 for i/o error, -2 for wrong sum, 0 on success */
 	char* allocated_ptr;
 
@@ -38,11 +37,12 @@ struct file_info {
 void file_info_destroy(struct file_info*); /* free allocated memory */
 const char* file_info_get_utf8_print_path(struct file_info*);
 
-int calculate_and_print_sums(FILE* out, const char *print_path, const char *full_path, struct rsh_stat_struct* stat_buf);
-int check_hash_file(const char* hash_file_path, int chdir);
+void save_torrent_to(const char* path, struct rhash_context* rctx);
+int calculate_and_print_sums(FILE* out, file_t* file, const char *print_path);
+int check_hash_file(file_t* file, int chdir);
 int rename_file_to_embed_crc32(struct file_info *info);
 void print_sfv_banner(FILE* out);
-int print_sfv_header_line(FILE* out, const char* printpath, const char* fullpath);
+int print_sfv_header_line(FILE* out, file_t* file, const char* printpath);
 
 #ifdef __cplusplus
 } /* extern "C" */
