@@ -10,6 +10,12 @@
 #include <time.h>
 #include <stdarg.h>
 #include <wchar.h>
+#include <errno.h>
+#include <sys/stat.h>
+
+#ifdef _WIN32
+#include <windows.h>
+#endif
 
 #include "win_utils.h"
 #include "parse_cmdline.h"
@@ -92,7 +98,6 @@ int urlencode(char *dst, const char *name)
 	if(!dst) {
 		int len;
 		for(len = 0; *name; name++) len += (IS_GOOD_URL_CHAR(*name) ? 1 : 3);
-		/* ed2k://|file|<fname>|<fsize>|2E398E5533AE4A83475B1AF001C6CEE6|h=RKLBEXT4O2H4RZER676WAVWGACIHQ56Z|/ */
 		return len;
 	}
 	/* encode URL as specified by RFC 1738 */
@@ -156,6 +161,25 @@ char* str_set(char* buf, int ch, int length)
 	return buf;
 }
 
+/**
+ * Concatenates two strings and returns allocated buffer with result.
+ *
+ * @param orig original string
+ * @param append the string to append
+ * @return the buffer
+ */
+char* str_append(const char* orig, const char* append)
+{
+	size_t len1 = strlen(orig);
+	size_t len2 = strlen(append);
+	char* res = (char*)rsh_malloc(len1 + len2 + 1);
+
+	/* concatenate two strings */
+	memcpy(res, orig, len1);
+	memcpy(res + len1, append, len2 + 1);
+	return res;
+}
+
 /**
  * Check if a string is a binary string, which means the string contain
  * a character with ACII code below 0x20 other than '\r', '\n', '\t'.
@@ -287,9 +311,10 @@ void print_time(FILE *out, time_t time)
 		t->tm_sec, (1900+t->tm_year), t->tm_mon+1, t->tm_mday);
 }
 
-#ifdef _WIN32
-#include <windows.h>
-#endif
+void print_time64(FILE *out, uint64_t time)
+{
+	print_time(out, (time_t)time);
+}
 
 /**
  * Return ticks in milliseconds for time intervals measurement.
@@ -309,7 +334,75 @@ unsigned rhash_get_ticks(void)
 #endif
 }
 
+/**
+ * Fill file information in the file_t structure.
+ *
+ * @param file the file information
+ * @param use_lstat nonzero if lstat() shall be used
+ * @return 0 on success, -1 on error
+ */
+int rsh_file_stat2(file_t* file, int use_lstat)
+{
+	struct rsh_stat_struct st;
+	int res = -1;
 
+#ifdef _WIN32
+	int i;
+	(void)use_lstat; /* ignore on windows */
+
+	if(file->wpath) {
+		free(file->wpath);
+		file->wpath = NULL;
+	}
+
+	for(i = 0; i < 2; i++) {
+		wchar_t* wpath = c2w(file->path, i);
+		if(wpath == NULL) continue;
+		res = clib_wstat(wpath, &st);
+		if(res == 0) {
+			file->wpath = wpath;
+			file->size  = st.st_size;
+
+			/* set correct filesize for large files under win32 */
+			win32_set_filesize64(file->path, &file->size);
+			break;
+		}
+		free(wpath);
+	}
+#else
+	res = (use_lstat ? lstat(file->path, &st) :
+		rsh_stat(file->path, &st));
+	file->size  = st.st_size;
+#endif /* _WIN32 */
+
+	file->mtime = st.st_mtime;
+	file->mode  = 0;
+	if(S_ISDIR(st.st_mode)) file->mode |= FILE_IFDIR;
+
+	return res;
+}
+
+/**
+ * Fill file information in the file_t structure.
+ *
+ * @param file the file information
+ * @return 0 on success, -1 on error
+ */
+int rsh_file_stat(file_t* file)
+{
+	return rsh_file_stat2(file, 0);
+}
+
+void rsh_file_cleanup(file_t* file)
+{
+	(void)file;
+#ifdef _WIN32
+	if(file->wpath) {
+		free(file->wpath);
+		file->wpath = NULL;
+	}
+#endif /* _WIN32 */
+}
 
 /* program exit and error reporting functions */
 
