@@ -3,7 +3,7 @@
 #define COMMON_FUNC_H
 
 /* internationalization support via gettext/libintl */
-#ifndef NO_GETTEXT
+#ifdef USE_GETTEXT
 # include <libintl.h>
 # define _(str) gettext(str)
 # ifdef _WIN32
@@ -13,7 +13,7 @@
 # endif /* _WIN32 */
 #else
 # define _(str) (str)
-#endif /* NO_GETTEXT */
+#endif /* USE_GETTEXT */
 
 /* use 64-bit off_t */
 #define _LARGEFILE64_SOURCE
@@ -46,15 +46,35 @@ int  is_binary_string(const char* str);
 char* str_tolower(const char* str);
 char* str_trim(char* str);
 char* str_set(char* buf, int ch, int size);
+char* str_append(const char* orig, const char* append);
 size_t strlen_utf8_c(const char *str);
 
 #define IS_DASH_STR(s) ((s)[0] == '-' && (s)[1] == '\0')
 #define IS_COMMENT(c) ((c) == ';' || (c) == '#')
 
+/* modes for file_t.mode */
+#define FILE_IFDIR   0x01
+#define FILE_IFLNK   0x02
+
+typedef struct file_t {
+	char* path;
+#ifdef _WIN32
+	wchar_t* wpath;
+#endif
+	uint64_t size;
+	uint64_t mtime;
+	unsigned mode;
+} file_t;
+
 /* file function */
 const char* get_basename(const char* path);
 char* get_dirname(const char* path);
 char* make_path(const char* dir, const char* filename);
+void print_time(FILE *out, time_t time);
+void print_time64(FILE *out, uint64_t time);
+int rsh_file_stat(file_t* file);
+int rsh_file_stat2(file_t* file, int use_lstat);
+void rsh_file_cleanup(file_t* file);
 
 #ifdef _WIN32
 # define IF_WINDOWS(code) code
@@ -98,13 +118,10 @@ typedef  char rsh_tchar;
 
 typedef struct rsh_stat_struct rsh_stat_buf;
 
-void print_time(FILE *out, time_t time);
 unsigned rhash_get_ticks(void);
-
 void  rhash_exit(int code);
 
 
-
 /* clever malloc with error detection */
 #define rsh_malloc(size) rhash_malloc(size, __FILE__, __LINE__)
 #define rsh_calloc(num, size) rhash_calloc(num, size, __FILE__, __LINE__)
