@@ -38,30 +38,6 @@ void rhash_hex_to_byte(const char* str, unsigned char* bin, int len)
 	}
 }
 
-#if 0
-/**
- * Parse given base32 string and store result to bin.
- *
- * @param str string to parse
- * @param bin result
- * @param len string length
- */
-void rhash_base32_to_byte(const char* str, unsigned char* bin, int len)
-{
-	const char* e = str + len;
-	unsigned shift = 0;
-	unsigned char b;
-	for(; str<e; str++) {
-		b = BASE32_TO_DIGIT(*str);
-		shift = (shift + 5) % 8;
-		if(shift < 5) {
-			*bin++ |= (b >> shift);
-		}
-		*bin |= b << (8 - shift);
-	}
-}
-#endif
-
 /**
  * Decode an URL-encoded string in the specified buffer.
  *
@@ -479,9 +455,9 @@ int hash_check_parse_line(char* line, hash_check* hashes, int check_eol)
 					if(FOURC2U('u', 'r', 'n', ':') != FOURC2U(hs.begin[-4],
 						hs.begin[-3], hs.begin[-2], hs.begin[-1])) return 0;
 
-					/* look for hash function by the name of magnet-parameter */
+					/* find hash by its magnet link specific URN name  */
 					for(i = 0; i < RHASH_HASH_COUNT; i++) {
-						const char* urn = hash_info_table[i].urn;
+						const char* urn = rhash_get_magnet_name(1 << i);
 						size_t len = hf_end - hs.begin;
 						if(strncmp(hs.begin, urn, len) == 0 &&
 							urn[len] == '\0') break;
@@ -500,7 +476,8 @@ int hash_check_parse_line(char* line, hash_check* hashes, int check_eol)
 					if(!hash_check_find_str(&hs, "\3")) bad = 1;
 					if(hs.begin != param_end) bad = 1;
 					break;
-					/* this switch() skips all unknown parameters */
+
+					/* note: this switch() skips all unknown parameters */
 				}
 			}
 			if(!bad && next) hs.begin = next;
