@@ -35,6 +35,8 @@
 #define RCTX_AUTO_FINAL 0x1
 #define RCTX_FINALIZED  0x2
 #define RCTX_FINALIZED_MASK (RCTX_AUTO_FINAL | RCTX_FINALIZED)
+#define RHPR_FORMAT (RHPR_RAW | RHPR_HEX | RHPR_BASE32 | RHPR_BASE64)
+#define RHPR_MODIFIER (RHPR_UPPERCASE | RHPR_REVERSE)
 
 /**
  * Initialize static data of rhash algorithms
@@ -279,7 +281,7 @@ RHASH_API int rhash_final(rhash ctx, unsigned char* first_result)
 	for(i = 0; i < ectx->hash_vector_size; i++) {
 		struct rhash_hash_info* info = ectx->vector[i].hash_info;
 		assert(info->final != 0);
-		assert(info->info.digest_size < sizeof(buffer));
+		assert(info->info->digest_size < sizeof(buffer));
 		info->final(ectx->vector[i].context, out);
 		out = buffer;
 	}
@@ -322,16 +324,16 @@ static void rhash_put_digest(rhash ctx, unsigned hash_id, unsigned char* result)
 			}
 			item = &ectx->vector[i];
 			info = item->hash_info;
-			if(info->info.hash_id == hash_id) break;
+			if(info->info->hash_id == hash_id) break;
 		}
 	}
 	digest = ((unsigned char*)item->context + info->digest_diff);
-	if(info->info.flags & F_SWAP32) {
-		rhash_u32_swap_copy(result, 0, digest, info->info.digest_size);
-	} else if(info->info.flags & F_SWAP64) {
-		rhash_u64_swap_copy(result, 0, digest, info->info.digest_size);
+	if(info->info->flags & F_SWAP32) {
+		rhash_u32_swap_copy(result, 0, digest, info->info->digest_size);
+	} else if(info->info->flags & F_SWAP64) {
+		rhash_u64_swap_copy(result, 0, digest, info->info->digest_size);
 	} else {
-		memcpy(result, digest, info->info.digest_size);
+		memcpy(result, digest, info->info->digest_size);
 	}
 }
 
@@ -496,13 +498,13 @@ RHASH_API int rhash_wfile(unsigned hash_id, const wchar_t* filepath, unsigned ch
  * @param hash_id the id of hash algorithm
  * @return pointer to the rhash_info structure containing the information
  */
-rhash_info* rhash_info_by_id(unsigned hash_id)
+const rhash_info* rhash_info_by_id(unsigned hash_id)
 {
 	hash_id &= RHASH_ALL_HASHES;
 	/* check that only one bit is set */
 	if(hash_id != (hash_id & -(int)hash_id)) return NULL;
 	/* note: alternative condition is (hash_id == 0 || (hash_id & (hash_id - 1)) != 0) */
-	return &(rhash_info_table[rhash_ctz(hash_id)].info);
+	return rhash_info_table[rhash_ctz(hash_id)].info;
 }
 
 /**
@@ -527,7 +529,7 @@ RHASH_API int rhash_get_digest_size(unsigned hash_id)
 {
 	hash_id &= RHASH_ALL_HASHES;
 	if(hash_id == 0 || (hash_id & (hash_id - 1)) != 0) return -1;
-	return (int)rhash_info_table[rhash_ctz(hash_id)].info.digest_size;
+	return (int)rhash_info_table[rhash_ctz(hash_id)].info->digest_size;
 }
 
 /**
@@ -538,7 +540,7 @@ RHASH_API int rhash_get_digest_size(unsigned hash_id)
  */
 RHASH_API int rhash_get_hash_length(unsigned hash_id)
 {
-	rhash_info* info = rhash_info_by_id(hash_id);
+	const rhash_info* info = rhash_info_by_id(hash_id);
 	return (int)(info ? (info->flags & F_BS32 ?
 		BASE32_LENGTH(info->digest_size) : info->digest_size * 2) : 0);
 }
@@ -551,10 +553,135 @@ RHASH_API int rhash_get_hash_length(unsigned hash_id)
  */
 RHASH_API const char* rhash_get_name(unsigned hash_id)
 {
-	rhash_info* info = rhash_info_by_id(hash_id);
+	const rhash_info* info = rhash_info_by_id(hash_id);
 	return (info ? info->name : 0);
 }
 
+/**
+ * Returns a name part of magnet urn of the given hash algorithm.
+ * Such magnet_name is used to generate a magnet link of the form
+ * urn:&lt;magnet_name&gt;=&lt;hash_value&gt;.
+ *
+ * @param hash_id the id of hash algorithm
+ * @return name
+ */
+RHASH_API const char* rhash_get_magnet_name(unsigned hash_id)
+{
+	const rhash_info* info = rhash_info_by_id(hash_id);
+	return (info ? info->magnet_name : 0);
+}
+
+static size_t rhash_get_magnet_url_size(const char* filepath,
+	rhash context, unsigned hash_mask, int flags)
+{
+	size_t size = 0; /* count terminating '\0' */
+	unsigned bit, hash = context->hash_id & hash_mask;
+
+	/* RHPR_NO_MAGNET, RHPR_FILESIZE */
+	if((flags & RHPR_NO_MAGNET) == 0) {
+		size += 8;
+	}
+
+	if((flags & RHPR_FILESIZE) != 0) {
+		uint64_t num = context->msg_size;
+
+		size += 4;
+		if(num == 0) size++;
+		else {
+			for(; num; num /= 10, size++);
+		}
+	}
+
+	if(filepath) {
+		size += 4 + rhash_urlencode(NULL, filepath);
+	}
+
+	/* loop through hash values */
+	for(bit = hash & -hash; bit <= hash; bit <<= 1) {
+		const char* name;
+		if((bit & hash) == 0) continue;
+		if((name = rhash_get_magnet_name(bit)) == 0) continue;
+
+		size += (7 + 2) + strlen(name);
+		size += rhash_print(NULL, context, bit,
+			(bit & (RHASH_SHA1 | RHASH_BTIH) ? RHPR_BASE32 : 0));
+	}
+
+	return size;
+}
+
+/**
+ * Print magnet link with given filepath and calculated hash sums into the
+ * output buffer. The hash_mask can limit which hash values will be printed.
+ * The function returns the size of the required buffer.
+ * If output is NULL the .
+ *
+ * @param output a string buffer to receive the magnet link or NULL
+ * @param filepath the file path to be printed or NULL
+ * @param context algorithms state
+ * @param hash_mask bit mask of the hash sums to add to the link
+ * @param flags   can be combination of bits RHPR_UPPERCASE, RHPR_NO_MAGNET,
+ *                RHPR_FILESIZE
+ * @return number of written characters, including terminating '\0' on success, 0 on fail
+ */
+RHASH_API size_t rhash_print_magnet(char* output, const char* filepath,
+	rhash context, unsigned hash_mask, int flags)
+{
+	int i;
+	const char* begin = output;
+
+	if(output == NULL) return rhash_get_magnet_url_size(
+		filepath, context, hash_mask, flags);
+
+	/* RHPR_NO_MAGNET, RHPR_FILESIZE */
+	if((flags & RHPR_NO_MAGNET) == 0) {
+		strcpy(output, "magnet:?");
+		output += 8;
+	}
+
+	if((flags & RHPR_FILESIZE) != 0) {
+		strcpy(output, "xl=");
+		output += 3;
+		output += rhash_sprintI64(output, context->msg_size);
+		*(output++) = '&';
+	}
+
+	if(filepath) {
+		strcpy(output, "dn=");
+		output += 3;
+		output += rhash_urlencode(output, filepath);
+		*(output++) = '&';
+	}
+	flags &= RHPR_UPPERCASE;
+
+	for(i = 0; i < 2; i++) {
+		unsigned bit;
+		unsigned hash = context->hash_id & hash_mask;
+		hash = (i == 0 ? hash & (RHASH_ED2K | RHASH_AICH)
+			: hash & ~(RHASH_ED2K | RHASH_AICH));
+		if(!hash) continue;
+
+		/* loop through hash values */
+		for(bit = hash & -hash; bit <= hash; bit <<= 1) {
+			const char* name;
+			if((bit & hash) == 0) continue;
+			if(!(name = rhash_get_magnet_name(bit))) continue;
+
+			strcpy(output, "xt=urn:");
+			output += 7;
+			strcpy(output, name);
+			output += strlen(name);
+			*(output++) = ':';
+			output += rhash_print(output, context, bit, 
+				(bit & (RHASH_SHA1 | RHASH_BTIH) ? flags | RHPR_BASE32 : flags));
+			*(output++) = '&';
+		}
+	}
+	output[-1] = '\0'; /* terminate the line */
+
+	return (output - begin);
+}
+
 /* hash sum output */
 
 /**
@@ -573,7 +700,7 @@ size_t rhash_print_bytes(char* output, const unsigned char* bytes,
 {
 	size_t str_len;
 	int upper_case = (flags & RHPR_UPPERCASE);
-	int format = (flags & ~(RHPR_UPPERCASE | RHPR_REVERSE));
+	int format = (flags & ~RHPR_MODIFIER);
 
 	switch(format) {
 	case RHPR_HEX:
@@ -606,31 +733,45 @@ size_t rhash_print_bytes(char* output, const unsigned char* bytes,
  * @param hash_id id of the hash sum to print or 0 to print the first hash
  *                saved in the context.
  * @param flags  controls how to print the sum, can contain flags
- *               RHPR_UPPERCASE, RHPR_HEX, RHPR_BASE32, RHPR_BASE64, e.t.c.
- * @return number of writen characters on success, 0 on fail
+ *               RHPR_UPPERCASE, RHPR_HEX, RHPR_BASE32, RHPR_BASE64, etc.
+ * @return number of written characters on success, 0 on fail
  */
 size_t RHASH_API rhash_print(char* output, rhash context, unsigned hash_id, int flags)
 {
-	rhash_info* info;
+	const rhash_info* info;
 	unsigned char digest[80];
 	size_t digest_size;
 
 	info = (hash_id != 0 ? rhash_info_by_id(hash_id) :
-		&((rhash_context_ext*)context)->vector[0].hash_info->info);
+		((rhash_context_ext*)context)->vector[0].hash_info->info);
 
 	if(info == NULL) return 0;
 	digest_size = info->digest_size;
 	assert(digest_size <= 64);
 
-	/* note: use info->hash_id, cause hash_id can be 0 */
-	rhash_put_digest(context, info->hash_id, digest);
-
-	/* use default text presentation if not specified by flags */
-	if((flags & ~(RHPR_UPPERCASE|RHPR_REVERSE)) == 0) {
+	flags &= (RHPR_FORMAT | RHPR_MODIFIER);
+	if((flags & RHPR_FORMAT) == 0) {
+		/* use default format if not specified by flags */
 		flags |= (info->flags & RHASH_INFO_BASE32 ? RHPR_BASE32 : RHPR_HEX);
 	}
 
-	if((flags & ~RHPR_UPPERCASE) == (RHPR_REVERSE|RHPR_HEX)) {
+	if(output == NULL) {
+		switch(flags & RHPR_FORMAT) {
+		case RHPR_HEX:
+			return (digest_size * 2);
+		case RHPR_BASE32:
+			return BASE32_LENGTH(digest_size);
+		case RHPR_BASE64:
+			return BASE64_LENGTH(digest_size);
+		default:
+			return digest_size;
+		}
+	}
+
+	/* note: use info->hash_id, cause hash_id can be 0 */
+	rhash_put_digest(context, info->hash_id, digest);
+
+	if((flags & ~RHPR_UPPERCASE) == (RHPR_REVERSE | RHPR_HEX)) {
 		/* reverse the digest */
 		unsigned char *p = digest, *r = digest + digest_size - 1;
 		char tmp;
@@ -680,22 +821,26 @@ static rhash_uptr_t process_bt_msg(unsigned msg_id, torrent_ctx* bt, rhash_uptr_
 
 	switch(msg_id) {
 	case RMSG_BT_ADD_FILE:
-		rhash_torrent_add_file(bt, (const char*)ldata, *(unsigned long long*)rdata);
+		bt_add_file(bt, (const char*)ldata, *(unsigned long long*)rdata);
 		break;
 	case RMSG_BT_SET_OPTIONS:
-		rhash_torrent_set_options(bt, (unsigned)ldata);
+		bt_set_options(bt, (unsigned)ldata);
 		break;
 	case RMSG_BT_SET_ANNOUNCE:
-		rhash_torrent_set_announce(bt, (const char*)ldata);
+		bt_set_announce(bt, (const char*)ldata);
 		break;
 	case RMSG_BT_SET_PIECE_LENGTH:
-		rhash_torrent_set_piece_length(bt, (size_t)ldata);
+		bt_set_piece_length(bt, (size_t)ldata);
+		break;
+	case RMSG_BT_SET_BATCH_SIZE:
+		bt_set_piece_length(bt,
+			bt_default_piece_length(*(unsigned long long*)ldata));
 		break;
 	case RMSG_BT_SET_PROGRAM_NAME:
-		rhash_torrent_set_program_name(bt, (const char*)ldata);
+		bt_set_program_name(bt, (const char*)ldata);
 		break;
 	case RMSG_BT_GET_TEXT:
-		return RHASH_STR2UPTR(rhash_torrent_get_text(bt, (char**)ldata));
+		return RHASH_STR2UPTR(bt_get_text(bt, (char**)ldata));
 	default:
 		return RHASH_ERROR; /* unknown message */
 	}
@@ -724,7 +869,7 @@ RHASH_API rhash_uptr_t rhash_transmit(unsigned msg_id, void* dst, rhash_uptr_t l
 			unsigned i;
 			for(i = 0; i < ctx->hash_vector_size; i++) {
 				struct rhash_hash_info* info = ctx->vector[i].hash_info;
-				if(info->info.hash_id == (unsigned)ldata)
+				if(info->info->hash_id == (unsigned)ldata)
 					return PVOID2UPTR(ctx->vector[i].context);
 			}
 			return (rhash_uptr_t)0;
@@ -761,6 +906,7 @@ RHASH_API rhash_uptr_t rhash_transmit(unsigned msg_id, void* dst, rhash_uptr_t l
 	case RMSG_BT_SET_PIECE_LENGTH:
 	case RMSG_BT_SET_PROGRAM_NAME:
 	case RMSG_BT_GET_TEXT:
+	case RMSG_BT_SET_BATCH_SIZE:
 		return process_bt_msg(msg_id, (torrent_ctx*)(((rhash_context_ext*)dst)->bt_ctx), ldata, rdata);
 
 	default:
