@@ -41,24 +41,24 @@ typedef struct torrent_ctx
 	int error; /* non-zero if error occurred, zero otherwise */
 } torrent_ctx;
 
-void rhash_torrent_init(torrent_ctx *ctx);
-void rhash_torrent_update(torrent_ctx *ctx, const void* msg, size_t size);
-void rhash_torrent_final(torrent_ctx *ctx, unsigned char result[20]);
-void rhash_torrent_cleanup(torrent_ctx *ctx);
+void bt_init(torrent_ctx *ctx);
+void bt_update(torrent_ctx *ctx, const void* msg, size_t size);
+void bt_final(torrent_ctx *ctx, unsigned char result[20]);
+void bt_cleanup(torrent_ctx *ctx);
 
-size_t rhash_torrent_get_text(torrent_ctx *ctx, char** pstr);
-unsigned char* rhash_torrent_get_btih(torrent_ctx *ctx);
+size_t bt_get_text(torrent_ctx *ctx, char** pstr);
+unsigned char* bt_get_btih(torrent_ctx *ctx);
 
 /* possible options */
 #define BT_OPT_PRIVATE 1
 #define BT_OPT_INFOHASH_ONLY 2
 
-void rhash_torrent_set_options(torrent_ctx *ctx, unsigned options);
-int  rhash_torrent_add_file(torrent_ctx *ctx, const char* path, uint64_t filesize);
-int  rhash_torrent_set_announce(torrent_ctx *ctx, const char* announce_url);
-int  rhash_torrent_set_program_name(torrent_ctx *ctx, const char* name);
-void rhash_torrent_set_piece_length(torrent_ctx *ctx, size_t piece_length);
-size_t rhash_torrent_default_piece_length(uint64_t total_size);
+void bt_set_options(torrent_ctx *ctx, unsigned options);
+int  bt_add_file(torrent_ctx *ctx, const char* path, uint64_t filesize);
+int  bt_set_announce(torrent_ctx *ctx, const char* announce_url);
+int  bt_set_program_name(torrent_ctx *ctx, const char* name);
+void bt_set_piece_length(torrent_ctx *ctx, size_t piece_length);
+size_t bt_default_piece_length(uint64_t total_size);
 
 #ifdef __cplusplus
 } /* extern "C" */
