@@ -38,7 +38,7 @@ static void print_help(void)
 		"Usage: %s [OPTION...] [FILE | -]...\n"
 		"       %s --printf=<format string> [FILE | -]...\n\n"), VERSION_STRING, CMD_FILENAME, CMD_FILENAME);
 
-	fprintf(rhash_data.out, "%s%s%s",
+	fprintf(rhash_data.out, "%s%s%s%s",
 		_("Options:\n"
 		"  -V, --version Print program version and exit.\n"
 		"  -h, --help    Print this help screen.\n"
@@ -59,8 +59,8 @@ static void print_help(void)
 		"      --has160  Calculate HAS-160 hash.\n"
 		"      --edonr256, --edonr512  Calculate EDON-R 256/512 hash.\n"
 		"      --snefru128, --snefru256  Calculate SNEFRU-128/256 hash.\n"
-		"  -a, --all     Calculate all supported hashes.\n"
-		"  -c, --check   Check hash files specified by command line.\n"
+		"  -a, --all     Calculate all supported hashes.\n"),
+		_("  -c, --check   Check hash files specified by command line.\n"
 		"  -u, --update  Update hash files specified by command line.\n"
 		"  -e, --embed-crc  Rename files by inserting crc32 sum into name.\n"
 		"      --check-embedded  Verify files by crc32 sum embedded in their names.\n"
@@ -178,6 +178,7 @@ static void accept_video(options_t *o)
 static void nya(void)
 {
 	fprintf(rhash_data.out, "  /\\__/\\\n (^ _ ^.) %s\n  (_uu__)\n",
+		/* TRANSLATORS: Keep secret ;) */
 		_("Purrr..."));
 	rsh_exit(0);
 }
@@ -338,6 +339,7 @@ cmdline_opt_t cmdline_opt[] =
 	{ F_UFLG,   0,   0, "bt-private", &opt.flags, OPT_BT_PRIVATE },
 	{ F_PFNC,   0,   0, "bt-piece-length", set_bt_piece_length, 0 },
 	{ F_CSTR,   0,   0, "bt-announce", &opt.bt_announce, 0 },
+	{ F_CSTR,   0,   0, "bt-batch", &opt.bt_batch_file, 0 },
 	{ F_UFLG,   0,   0, "benchmark-raw", &opt.flags, OPT_BENCH_RAW },
 	{ F_PFNC,   0,   0, "openssl", openssl_flags, 0 },
 
@@ -869,7 +871,7 @@ void options_destroy(struct options_t* o)
 
 /**
  * Check that options do not conflict with each other.
- * Also do some final options processing steps.
+ * Also make some final options processing steps.
  */
 static void make_final_options_checks(void)
 {
@@ -880,6 +882,9 @@ static void make_final_options_checks(void)
 		log_msg(_("Config file: %s\n"), (conf_opt.config_file ? conf_opt.config_file : _("None")));
 	}
 
+	if(opt.bt_batch_file) opt.mode |= MODE_TORRENT;
+	if(opt.mode & MODE_TORRENT) opt.sum_flags |= RHASH_BTIH;
+
 	/* check that no more than one program mode specified */
 	if(opt.mode & (opt.mode - 1)) {
 		die(_("incompatible program modes\n"));
@@ -890,8 +895,6 @@ static void make_final_options_checks(void)
 		die(_("too many formating options\n"));
 	}
 
-	if(opt.mode & MODE_TORRENT) opt.sum_flags |= RHASH_BTIH;
-
 	if(!opt.crc_accept) opt.crc_accept = file_mask_new_from_list(".sfv");
 
 	if(opt.openssl_mask) rhash_transmit(RMSG_SET_OPENSSL_MASK, 0, opt.openssl_mask, 0);
