@@ -73,9 +73,10 @@ struct options_t {
 	int   find_max_depth;
 	struct vector_t *files_accept; /* suffixes of files for which sums will be calculated */
 	struct vector_t *crc_accept;   /* suffixes of crc files to verify or update */
-	size_t bt_piece_length; /* BitTorrent piece length */
-	char*  bt_announce;
 	unsigned openssl_mask;  /* mask which openssl hashes to use */
+	size_t bt_piece_length; /* BitTorrent piece length */
+	char*  bt_announce;     /* BitTorrent announce url */
+	char*  bt_batch_file;   /* path to save batch torrent to */
 
 	char** argv;
 	char** files; /* the files to process */
